# Dedi Sylius Customer Pool Plugin

This plugin for [Sylius](https://sylius.com/) allows you to manage customer pools.

Supports Doctrine ORM driver only.

Customer Pool is a collection of Customers that is assigned to a specific channel. Thanks to this concept, if you have two channels, each of them has a separate customer pool, then customers that have accounts in channel A, and have not registered in channel B, will not be able to log in to channel B with credentials they have specified in channel A (which is the behaviour happening in Sylius open source edition). This feature allows you to sell via multiple channels, creating a illusion of shopping in completely different stores, while you still have one administration panel.

## Installation

### Step 1: Install the plugin

Open a command console, enter your project directory and execute the following command to download the latest stable version of this plugin:

```bash
$ composer require dedi/sylius-customer-pool-plugin
```

This command requires you to have Composer installed globally, as explained in the [installation chapter](https://getcomposer.org/doc/00-intro.md) of the Composer documentation.

### Step 2: Enable the plugin

Then, enable the plugin by adding it to the list of registered plugins/bundles in `config/bundles.php` file of your project:

```php
<?php
$bundles = [
    Dedi\SyliusCustomerPoolPlugin\DediSyliusCustomerPoolPlugin::class => ['all' => true],
];
```

### Step 3: Configure the plugin
```yaml
# config/packages/dedi_sylius_customer_pool.yaml

imports:
    - { resource: "@DediSyliusCustomerPoolPlugin/Resources/config/app/config.yaml" }
```

```yaml
# config/routes/dedi_sylius_customer_pool.yaml

dedi_sylius_customer_pool_admin:
    resource: "@DediSyliusCustomerPoolPlugin/Resources/config/routing.yaml"
```

```yaml
# config/validator/validation.yaml
imports:
  - { resource: "@DediSyliusCustomerPoolPlugin/Resources/config/validation/CustomerPool.xml" }
  - { resource: "@DediSyliusCustomerPoolPlugin/Resources/config/validation/Customer.xml" }
```

#### Customize your Customer and Channel model

Add a `Dedi\SyliusCustomerPoolPlugin\Model\Customer\CustomerPoolTrait` trait to your `App\Entity\Customer\Customer` and `App\Entity\Customer\Channel` class,
and implement `Dedi\SyliusCustomerPoolPlugin\Model\Customer\CustomerPoolAwareInterface`

In Customer entity, don't forget to override attributes to remove unique fields on email and canonical, and add a composite unique index.

```php
<?php 
// src/Entity/Customer/Customer.php

namespace App\Entity\Customer;

use Doctrine\ORM\Mapping as ORM;
use Sylius\Component\Core\Model\Customer as BaseCustomer;
use Dedi\SyliusCustomerPoolPlugin\Model\Customer\CustomerPoolAwareInterface;
use Dedi\SyliusCustomerPoolPlugin\Model\Customer\CustomerPoolTrait;
  
/**
 * @ORM\Entity
 * @ORM\Table(name="sylius_customer", uniqueConstraints={
*       @ORM\UniqueConstraint(name="UNQ_email_customer_pool", columns={"email", "customer_pool"}),
*       @ORM\UniqueConstraint(name="UNQ_email_canonical_customer_pool", columns={"email_canonical", "customer_pool"})
*  })
 * @ORM\AttributeOverrides({
 *      @ORM\AttributeOverride(name="email",
 *          column=@ORM\Column(
 *              name     = "email",
 *              type     = "string",
 *              nullable = false,
 *              unique   = false
 *          )
 *      ),
 *      @ORM\AttributeOverride(name="emailCanonical",
 *          column=@ORM\Column(
 *              name     = "email_canonical",
 *              type     = "string",
 *              nullable = false,
 *              unique   = false
 *          )
 *      )
 * })
 */
class Customer extends BaseCustomer implements CustomerPoolAwareInterface
{
    use CustomerPoolTrait;
}
```
```php
<?php 
// src/Entity/Customer/Customer.php

namespace App\Entity\Channel;

use Doctrine\ORM\Mapping as ORM;
use Sylius\Component\Core\Model\Channel as BaseChannel;
use Dedi\SyliusCustomerPoolPlugin\Model\Customer\CustomerPoolAwareInterface;
use Dedi\SyliusCustomerPoolPlugin\Model\Customer\CustomerPoolTrait;
  
/**
 * @ORM\Entity
 * @ORM\Table(name="sylius_channel")
 */
class Channel extends BaseChannel implements CustomerPoolAwareInterface
{
    use CustomerPoolTrait;
}
```

#### Customize your User repository

Add a `Dedi\SyliusCustomerPoolPlugin\Repository\FindUserByEmailAndCustomerPoolTrait` trait to your `App\Repository\User\UserRepository` class,
and implement `Dedi\SyliusCustomerPoolPlugin\Repository\UserWithCustomerPoolRepositoryInterface`

```php
<?php

declare(strict_types=1);

namespace App\Repository\User\UserRepository;

use Dedi\SyliusCustomerPoolPlugin\Repository\FindUserByEmailAndCustomerPoolTrait;
use Dedi\SyliusCustomerPoolPlugin\Repository\UserWithCustomerPoolRepositoryInterface;
use Sylius\Bundle\CoreBundle\Doctrine\ORM\UserRepository as BaseRepository;

class UserRepository extends BaseRepository implements UserWithCustomerPoolRepositoryInterface
{
    use FindUserByEmailAndCustomerPoolTrait;
}

```

### Step 5: Update your database schema

```bash
$ php bin/console doctrine:migrations:diff
$ php bin/console doctrine:migrations:migrate
```

### Step 6: Add EmailAndCustomerPoolProvider to security.yaml

This will restrict the user to login to a Channel having different Customer Pool setup.

```yaml
security:
    providers:
        sylius_shop_user_provider:
            id: dedi_sylius_customer_pool.provider.email_and_customer_pool
        sylius_api_shop_user_provider:
            id: dedi_sylius_customer_pool.provider.email_and_customer_pool
```